
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <stdlib.h>


#include "msgParser.h"
#include "GPSNovatel.h"


// DEBUG
// #define DEBUG_ULONG
// #define DEBUG_USHORT
// #define DEBUG_HEADER

// #define DEBUG_HEADER
// #define DEBUG_BESTPOSB

/**
 ** @Name : 
 ** @Description : 
 **
 **
 ** @parameter:
 **
 **
 **/
void computeBinaryHeader(unsigned char *ucTrame, int *offset,OEM4_BINARY_HEADER *BINARY_HEADER)
{	

//  printf("offset depart  = %d\n", *offset);

  // le premier sync est forcement = 0xAA
  BINARY_HEADER->sop1 = 0xAA;
	
  // 0x44 		
  BINARY_HEADER->sop2 = readCharSerialNext(ucTrame, offset);
	
  // 0x12 
  BINARY_HEADER->sop3 = readCharSerialNext(ucTrame, offset);
	
  // Header lenght
  BINARY_HEADER->header_length = readUcharSerialNext(ucTrame, offset);
  
  // Message ID
  BINARY_HEADER->number = readUshortSerialNext(ucTrame, offset);
  
  // Message Type
  BINARY_HEADER->type = readCharSerialNext(ucTrame, offset);
   
  // Traitement de la trame
  // Port address
  BINARY_HEADER->port_address = readUcharSerialNext(ucTrame, offset);
  
  // Message length
  BINARY_HEADER->length = readUshortSerialNext(ucTrame, offset);
  
  // Sequence
  BINARY_HEADER->sequence = readUshortSerialNext(ucTrame, offset);
  
  // Idle time
  BINARY_HEADER->idle = readUcharSerialNext(ucTrame, offset);
  
  // GPS Time Status
  BINARY_HEADER->gps_stat = readUcharSerialNext(ucTrame, offset);
  
  // GPS Week number
  BINARY_HEADER->gps_week = readUshortSerialNext(ucTrame, offset);
  
  // Milliseconds into week
  BINARY_HEADER->millisecs = readUlongSerialNext(ucTrame, offset);
  
  // Receiver status word
  BINARY_HEADER->status = readUlongSerialNext(ucTrame, offset);
  
  // Receiver internal Use
  BINARY_HEADER->internalUse = readUshortSerialNext(ucTrame, offset);
  
  // Receiver software version
  BINARY_HEADER->version = readUshortSerialNext(ucTrame, offset);
   	
#ifdef DEBUG_HEADER
  printf("\n\n START OF BINARY HEADER\n");
  printf("Sync 1 = %#0x \n", BINARY_HEADER->sop1);
  printf("Sync 2 = %#0x \n", BINARY_HEADER->sop2);
  printf("Sync 3 = %#0x \n", BINARY_HEADER->sop3);
  printf("Header lenght = %#0x \n", BINARY_HEADER->header_length);
  printf("MESSAGE ID = %#hx \n", BINARY_HEADER->number);
  printf("MESSAGE TYPE = %#0x \n", BINARY_HEADER->type);
  printf("Port address = %#0x \n", BINARY_HEADER->port_address);
  printf("MESSAGE LENGTH = %#hx \n", BINARY_HEADER->length);
  printf("Sequence = %#hx \n", BINARY_HEADER->sequence);
  printf("Idle time = %#0x \n", BINARY_HEADER->idle);
  printf("GPS Time status = %#0x \n", BINARY_HEADER->gps_stat);
  printf("GPS Week number = %#hx \n", BINARY_HEADER->gps_week);
  printf("Milliseconds into week = %#lx \n", BINARY_HEADER->millisecs);
  printf("Receiver status word = %#0x \n", BINARY_HEADER->status);
  printf("Receiver software version = %#hx \n", BINARY_HEADER->version);			
  printf("\nEND OF BINARY HEADER\n");
#endif    	 
}

/**
 * Cette fonction route les trames en fonction de leur ID
 *
 * @param
 * @param 
 * @param 
 * @param 
 * @return void
 */
void computeBinaryMessage(unsigned char *ucTrame, int offset, int HEADER_LENGTH,
			  int MESSAGE_LENGTH, int indice, struct timeval timev)
{
  // Structure de donnes pour stocker le Header binaire uniquement
  // pour chaque "indice" on stock la trame binaire
  computeBinaryHeader(ucTrame, &offset, &USR_TRAME[indice].BINARY_HEADER);

  // Horodatage de la trame, on ecrit dans la structure de stockage l'horodatage
  // de la trame
  USR_TRAME[indice].timev.tv_sec = timev.tv_sec;
  USR_TRAME[indice].timev.tv_usec = timev.tv_usec;

  // On dtrame le message reu en le stockant dans le tableau  la position "indice"
  switch(USR_TRAME[indice].BINARY_HEADER.number)
    {
    case BESTPOSb: // traitement BESTPOSb     
      computeBESTPOSb( &USR_TRAME[indice].TRAME_TYPE.strOEM4BESTPOSb_LOG, ucTrame, &offset );     
      break;

    case BESTUTMb: // traitement BESTUTM   
      computeBESTUTMb( &USR_TRAME[indice].TRAME_TYPE.strOEM4BESTUTMb_LOG, ucTrame, &offset );
      break;

    case BESTXYZb: // traitement BESTXYZb
      computeBESTXYZb( &USR_TRAME[indice].TRAME_TYPE.strOEM4BESTXYZb_LOG, ucTrame, &offset );
      break;

    case BESTVELb:
      computeBESTVELb( &USR_TRAME[indice].TRAME_TYPE.strOEM4BESTVELb_LOG, ucTrame, &offset );
      break;

    default:
     // printf("\ntrame inconnue\n");
      break;
    }
}


void computeBESTVELb(OEM4BESTVELb_LOG  *strBESTVEL, unsigned char *ucTrame, int *offset)
{
    strBESTVEL->solStatus = readUlongSerialNext(ucTrame, offset);
    strBESTVEL->velocityType = readUlongSerialNext(ucTrame, offset);
    strBESTVEL->latency = readFloatSerialNext(ucTrame, offset);
    strBESTVEL->age = readFloatSerialNext(ucTrame, offset);
    strBESTVEL->horizontalSpeed = readDoubleSerialNext(ucTrame, offset);
    strBESTVEL->actualDirection = readDoubleSerialNext(ucTrame, offset);
    strBESTVEL->verticalSpeed = readDoubleSerialNext(ucTrame, offset);
}

/**
 * 
 *
 * 
 *
 * 
 *  
 * 
 * 
 */
void computeBESTPOSb(OEM4BESTPOSb_LOG  *strBESTPOS, unsigned char *ucTrame, int *offset)
{    
  // Solution status
  strBESTPOS->sol_stat = readUlongSerialNext(ucTrame, offset);

  // Position type
  strBESTPOS->pos_type = readUlongSerialNext(ucTrame, offset);

  // Latitude
  strBESTPOS->lat = readDoubleSerialNext(ucTrame, offset);

  // Longitude
  strBESTPOS->lon = readDoubleSerialNext(ucTrame, offset);

  // Height above mean sea level
  strBESTPOS->hgt = readDoubleSerialNext(ucTrame, offset);

  // Undulation
  strBESTPOS->und = readFloatSerialNext(ucTrame, offset);

  // Datum ID
  strBESTPOS->datum_id = readUlongSerialNext(ucTrame, offset);
 
  // Latitude standard deviation
  strBESTPOS->latdev = readFloatSerialNext(ucTrame, offset);

  // Longitude standard deviation
  strBESTPOS->londev = readFloatSerialNext(ucTrame, offset);

  // Height standard deviation
  strBESTPOS->hgtdev = readFloatSerialNext(ucTrame, offset);

  // Reference station ID
  strBESTPOS->ref_station_id[0] = readCharSerialNext(ucTrame, offset);
  strBESTPOS->ref_station_id[1] = readCharSerialNext(ucTrame, offset);
  strBESTPOS->ref_station_id[2] = readCharSerialNext(ucTrame, offset);
  strBESTPOS->ref_station_id[3] = readCharSerialNext(ucTrame, offset);

  // Differential correction lag
  strBESTPOS->lag = readFloatSerialNext(ucTrame, offset);

  // Solution age
  strBESTPOS->age = readFloatSerialNext(ucTrame, offset);

  // Number of matched satel
  strBESTPOS->sats = readUcharSerialNext(ucTrame, offset);

  // Number of GPS L1 ranges used
  strBESTPOS->gps_l1_ranges = readUcharSerialNext(ucTrame, offset);

  // Number of GPS L1 ranges used
  strBESTPOS->gps_l1_mask_ranges = readUcharSerialNext(ucTrame, offset);

  // Number of GPS L2 ranges used 
  strBESTPOS->gps_l2_mask_ranges = readUcharSerialNext(ucTrame, offset);

  // Reserved
  readUcharSerialNext(ucTrame, offset);
  readUcharSerialNext(ucTrame, offset);
  readUcharSerialNext(ucTrame, offset);
  readUcharSerialNext(ucTrame, offset);

#ifdef DEBUG_BESTPOSB
  printf("\n\nBESTPOSb\n");
  printf("Solution status = %hx\n",strBESTPOS->sol_stat);
  printf("Position type = %hx\n",strBESTPOS->pos_type);
  printf("Latitude = %.15g\n",strBESTPOS->lat);
  printf("Longitude =%.15g\n",strBESTPOS->lon);
  printf("Height above mean sea level  = %.15g\n",strBESTPOS->hgt);
  printf("Undulation  = %f\n",strBESTPOS->und);
  printf("Datum ID  = %ld\n",strBESTPOS->datum_id);
  printf("Latitude standard deviation  = %.10f\n",strBESTPOS->latdev);
  printf("Longitude standard deviation  = %.10f\n",strBESTPOS->londev);
  printf("Height standard deviation  = %.10f\n",strBESTPOS->hgtdev);
  printf("ID = %#0xh %#0xh %#0xh %#0xh \n",strBESTPOS->ref_station_id[0],
	 strBESTPOS->ref_station_id[1],
	 strBESTPOS->ref_station_id[2],
	 strBESTPOS->ref_station_id[3]);
  printf("Differential correction lag  = %.10f\n",strBESTPOS->lag );
  printf("Solution age  = %.5f\n",strBESTPOS->age);
  printf("Number of matched satellites = %d\n",strBESTPOS->sats);
  printf("Number of GPS L1 ranges used = %d\n",strBESTPOS->gps_l1_ranges);
  printf("Number of GPS L1 ranges used mask  = %d\n",strBESTPOS->gps_l1_mask_ranges);
  printf("Number of GPS L2 ranges used   = %d\n",strBESTPOS->gps_l2_mask_ranges);
  printf("FIN MESSAGE\n");
#endif
}


/**
 * 
 *
 * 
 *
 * 
 *  
 * 
 * 
 */
void computeBESTUTMb(OEM4BESTUTMb_LOG  *strBESTUTM, unsigned char *ucTrame, int *offset)
{
 
  // Traitement des donnes de la trame 

  // Solution status
  strBESTUTM->solStatus = readUlongSerialNext(ucTrame, offset); 

  // Position type           
  strBESTUTM->posType = readUlongSerialNext(ucTrame, offset);  

  // Longitudinal zone number            
  strBESTUTM->longitudZoneNumber = readUlongSerialNext(ucTrame, offset);
   
  // Latitudinal zone letter
  strBESTUTM->latitudinalZoneLetter =  readUlongSerialNext(ucTrame, offset);

  // Northing origin
  strBESTUTM->northing =  readDoubleSerialNext(ucTrame, offset);

  // Easting origin
  strBESTUTM->easting =  readDoubleSerialNext(ucTrame, offset);

  // Height above mean sea level
  strBESTUTM->height =  readDoubleSerialNext(ucTrame, offset);

  // Undulation 
  strBESTUTM->undulation =  readFloatSerialNext(ucTrame, offset);

  // Datum ID number
  strBESTUTM->datumID = readUlongSerialNext(ucTrame, offset); 

  // Northing st. deviation
  strBESTUTM->northingDev = readFloatSerialNext(ucTrame, offset);

  // Easting st. deviation
  strBESTUTM->eastingDev = readFloatSerialNext(ucTrame, offset);

  // Height st. deviation
  strBESTUTM->heightDev = readFloatSerialNext(ucTrame, offset);

  // Base station ID
  strBESTUTM->baseStationID[0] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->baseStationID[1] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->baseStationID[2] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->baseStationID[3] =  readCharSerialNext(ucTrame, offset);

  // Differential age in seconds
  strBESTUTM->differentialAge = readFloatSerialNext(ucTrame, offset);

  // Solution age in seconds
  strBESTUTM->solutionAge= readFloatSerialNext(ucTrame, offset);

  // Number of satellite tracked
  strBESTUTM->nbSatTracked =  readUcharSerialNext(ucTrame, offset);

  // Number of GPS L1 ranges used in computation
  strBESTUTM->nbGPSL1Used =  readUcharSerialNext(ucTrame, offset);

  // Number of GPS L1 ranges above RTK mask angle
  strBESTUTM->nbGPSL1 =  readUcharSerialNext(ucTrame, offset);

  // Number of GPS L2 ranges above RTK mask angle
  strBESTUTM->nbGPSL2 =  readUcharSerialNext(ucTrame, offset);

  // reserved 
  strBESTUTM->reserved[0] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->reserved[1] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->reserved[2] =  readCharSerialNext(ucTrame, offset);
  strBESTUTM->reserved[3] =  readCharSerialNext(ucTrame, offset);

#ifdef DEBUG_BESTUTMb
  printf("\n\nMESSAGE BESTUTMb\n");
  printf("Solution status = %u\n",strBESTUTM->solStatus);
  printf("Position type = %u\n",strBESTUTM->posType);
  printf("Longitudinal zone number  = %u\n",strBESTUTM->longitudZoneNumber);
  printf("Latitudinal zone letter = %c\n",strBESTUTM->latitudinalZoneLetter);
  printf("Northing origin = %.15f\n",strBESTUTM->northing);
  printf("Easting origin = %.15f\n",strBESTUTM->easting);
  printf("Height above mean sea level = %.15f\n",strBESTUTM->height);
  printf("Northing st. deviation = %.15f\n",strBESTUTM->northingDev);
  printf("Height st. deviation = %.15f\n",strBESTUTM->heightDev);
  printf("Easting origin = %.15f\n",strBESTUTM->eastingDev);
  printf("Undulation = %.15f\n",strBESTUTM->undulation);
  printf("Datum ID number = %u\n",strBESTUTM->datumID);
  printf("Base station ID.0 = %u\n",strBESTUTM->baseStationID[0]);
  printf("Base station ID.1 = %u\n",strBESTUTM->baseStationID[1]);
  printf("Base station ID.2 = %u\n",strBESTUTM->baseStationID[2]);
  printf("Base station ID.3 = %u\n",strBESTUTM->baseStationID[3]);
  printf("Differential age in seconds = %.15f\n",strBESTUTM->differentialAge);
  printf("Solution age in seconds = %.15f\n",strBESTUTM->solutionAge);
  printf("Number of satellite tracked = %u\n",strBESTUTM->nbSatTracked);
  printf("Number GPS L1 ranges used in computation = %u\n",strBESTUTM->nbGPSL1Used);
  printf("Number of GPS L1 ranges above RTK mask angle = %u\n",strBESTUTM->nbGPSL1);
  printf("Number of GPS L2 ranges above RTK mask angle = %u\n",strBESTUTM->nbGPSL2);
  printf("FIN MESSAGE\n");
#endif
}





void computeBESTXYZb(OEM4BESTXYZb_LOG *strBESTXYZb, unsigned char *ucTrame, int *offset)
{ 
  // Traitement des donnes de la trame 

  // Solution status
  strBESTXYZb->solStatus = readUlongSerialNext(ucTrame, offset); 
  
  // Position type
  strBESTXYZb->positionType  = readUlongSerialNext (ucTrame, offset); 

  // Position X
  strBESTXYZb->positionX =readDoubleSerialNext (ucTrame, offset); 
  
  // Position Y
  strBESTXYZb->positionY  =readDoubleSerialNext (ucTrame, offset); 
  
  // Position Z
  strBESTXYZb->positionZ  =readDoubleSerialNext (ucTrame, offset); 
  
  // Standard deviation pos. X
  strBESTXYZb->stdPositionX  =readFloatSerialNext (ucTrame, offset); 
  
  // Standard deviation pos. Y
  strBESTXYZb->stdPositionY  = readFloatSerialNext(ucTrame, offset); 
  
  // Standard deviation pos. Z
  strBESTXYZb->stdPositionZ  = readFloatSerialNext(ucTrame, offset); 
  
  // Solution status 
  strBESTXYZb->vSolStatus  = readUlongSerialNext(ucTrame, offset); 
  
  // Velocity status
  strBESTXYZb->velStatus  = readUlongSerialNext(ucTrame, offset); 
  
  // Velocity vector X
  strBESTXYZb->velocityVectorX  = readDoubleSerialNext(ucTrame, offset); 
  
  // Velocity vector Y
  strBESTXYZb->velocityVectorY  = readDoubleSerialNext(ucTrame, offset); 
  
  // Velocity vector Z
  strBESTXYZb->velocityVectorZ  = readDoubleSerialNext(ucTrame, offset); 
  
  // std. deviation Velocity vector X
  strBESTXYZb->stdDevVelocityVectorX  = readFloatSerialNext(ucTrame, offset); 
  
  // std. deviation Velocity vector Y
  strBESTXYZb->stdDevVelocityVectorY  = readFloatSerialNext(ucTrame, offset); 
  
  // std. deviation Velocity vector Z
  strBESTXYZb->stdDevVelocityVectorZ  = readFloatSerialNext(ucTrame, offset); 
  
  // Base ID
  strBESTXYZb-> baseID[0]   = readCharSerialNext(ucTrame, offset); 
  strBESTXYZb-> baseID[1]   = readCharSerialNext(ucTrame, offset); 
  strBESTXYZb-> baseID[2]   = readCharSerialNext(ucTrame, offset); 
  strBESTXYZb-> baseID[3]   = readCharSerialNext(ucTrame, offset); 
  
  // Velocity latency
  strBESTXYZb->velocityLatency  = readFloatSerialNext(ucTrame, offset); 
  
  // Differential age in s
  strBESTXYZb->differentialAge  = readFloatSerialNext(ucTrame, offset); 

  // Solution age in s
  strBESTXYZb->solutionAge  = readFloatSerialNext(ucTrame, offset); 
  
  // Number of observations tracked
  strBESTXYZb->nbObsTracked  = readUcharSerialNext(ucTrame, offset); 
  
  // Nb GPS L1 used in computation
  strBESTXYZb->nbGPSL1Used  = readUcharSerialNext(ucTrame, offset); 
  
  // Nb of GPS L1 range above RTK mask angle
  strBESTXYZb->nbGPSL1RTK  = readUcharSerialNext(ucTrame, offset); 
  
  // Nb of GPS L2 range above RTK mask angle
  strBESTXYZb->nbGPSL2RTK  = readUcharSerialNext(ucTrame, offset); 
  
  // Reserved
  strBESTXYZb->reserved[0] = readUcharSerialNext(ucTrame, offset); 
  strBESTXYZb->reserved[1] = readUcharSerialNext(ucTrame, offset); 
  strBESTXYZb->reserved[2] = readUcharSerialNext(ucTrame, offset); 
  strBESTXYZb->reserved[3] = readUcharSerialNext(ucTrame, offset); 

#ifdef DEBUG_BESTXYZb
  printf("\n\nMESSAGE BESTXYZb\n");
  printf("Solution status = %u\n",strBESTXYZb->solStatus);
  printf("Position type = %u\n",strBESTXYZb->positionType);
  printf("Position X  = %.15g\n",strBESTXYZb->positionX);
  printf("Position Y  = %.15g\n",strBESTXYZb->positionY);
  printf("Position Z  = %.15g\n",strBESTXYZb->positionZ);
  printf("Standard deviation pos. X  = %.15g\n",strBESTXYZb->stdPositionX);
  printf("Standard deviation pos. Y  = %.15g\n",strBESTXYZb->stdPositionY);
  printf("Standard deviation pos. Z  = %.15g\n",strBESTXYZb->stdPositionZ);
  printf("Solution status  = %u\n",strBESTXYZb->vSolStatus);
  printf("Velocity status  = %u\n",strBESTXYZb->velStatus);
  printf("Velocity vector X  = %.15g\n",strBESTXYZb->velocityVectorX);
  printf("Velocity vector Y  = %.15g\n",strBESTXYZb->velocityVectorY);
  printf("Velocity vector Z  = %.15g\n",strBESTXYZb->velocityVectorZ);
  printf("std. deviation Velocity vector X  = %.15g\n",
	 strBESTXYZb->stdDevVelocityVectorX);
  printf("std. deviation Velocity vector Y  = %.15g\n",
	 strBESTXYZb->stdDevVelocityVectorY);
  printf("std. deviation Velocity vector Z  = %.15g\n",
	 strBESTXYZb->stdDevVelocityVectorZ);
  printf("Base ID  = %u\n",strBESTXYZb->baseID[0]);
  printf("Base ID  = %u\n",strBESTXYZb->baseID[1]);
  printf("Base ID  = %u\n",strBESTXYZb->baseID[2]);
  printf("Base ID  = %u\n",strBESTXYZb->baseID[3]);
  printf("Velocity latency  = %.15g\n",strBESTXYZb->velocityLatency);
  printf("Differential age in s  = %.15g\n",strBESTXYZb->differentialAge);
  printf("Solution age in s  = %.15g\n",strBESTXYZb->differentialAge);
  printf("Number of observations tracked = %u\n",strBESTXYZb->nbObsTracked);
  printf("Nb GPS L1 used in computation = %u\n",strBESTXYZb->nbGPSL1Used);
  printf("Nb of GPS L1 range above RTK mask angle  = %u\n",strBESTXYZb->nbGPSL1RTK);
  printf("Nb of GPS L2 range above RTK mask angle  = %u\n",strBESTXYZb->nbGPSL2RTK);
  printf("FIN MESSAGE\n");
#endif
}



	
/**
 ** 
 **
 ** 
 **
 ** 
 ** 
 ** 
 ** 
 **/
unsigned char readUcharSerialNext(unsigned char *ucTrame, int *offset)
{
  unsigned char tmp = ucTrame[++*offset];

#ifdef DEBUG_UCHAR  
  printf("U CHAR = %#0x\n", tmp);
  printf("  offset = %d\n", *offset);
#endif
  
  return tmp;
}
	
/*!
  
 
  
 
  
  
  
  
*/
char readCharSerialNext(unsigned char *ucTrame, int *offset)
{
  char field;
 	 
  field = ucTrame[++*offset];


#ifdef DEBUG_CHAR 
  printf("CHAR = %#0x\n", field);
  printf("  offset = %d\n", *offset);
#endif

  return field;
}
	
	
/*!
  
 
  
 
  
  
  
  
*/
unsigned short readUshortSerialNext(unsigned char *ucTrame, int *offset)
{
  unsigned short destination;

  ((char*)&destination)[0] = ucTrame[++*offset];
  ((char*)&destination)[1] = ucTrame[++*offset];
 
#ifdef DEBUG_USHORT	
  printf("USHORT = %#hx, valeur decimal = %u\n",destination, destination);
  printf("  offset = %d\n", *offset);
#endif    

  return destination;	
}
	
/**
 ** 
 **
 ** 
 **
 ** 
 ** 
 ** 
 ** 
 **/
unsigned long readUlongSerialNext(unsigned char *ucTrame, int *offset)
{
  unsigned long destination;
	
  ((char*)&destination)[0] = ucTrame[++*offset];

  ((char*)&destination)[1] = ucTrame[++*offset];
  
  ((char*)&destination)[2] = ucTrame[++*offset];
  
  ((char*)&destination)[3] = ucTrame[++*offset];
 
#ifdef DEBUG_ULONG
  printf("ULONG = %#0x, %#0x, %#0x, %#0x => %#0x\n", 
	 ((char*)&destination)[0],
	 ((char*)&destination)[1],
	 ((char*)&destination)[2],
	 ((char*)&destination)[3],
	 destination);
  printf("  offset = %d\n", *offset);
#endif

  return destination;
}


/**
 * 
 *
 * 
 *
 * 
 * 
 * 
 * 
 */
double readDoubleSerialNext(unsigned char *ucTrame, int *offset)
{
  double destination;
  int i=0;
  unsigned char octet;
  
  for(i=0;i<8;i++)
    {    
      ((char*)&destination)[i] =  readUcharSerialNext(ucTrame, offset);
      
#ifdef DEBUG_DOUBLE  
      printf("DOUBLE OCTET %d = %#0x\n", i, destination)[i]); 
#endif
}

#ifdef DEBUG_DOUBLE
printf("####################################### RESULTAT DOUBLE =>  %.15g\n",destination);
printf("  offset = %d\n", *offset);
#endif	

return destination;	
}

/**
 * 
 *
 * 
 *
 * 
 * 
 * 
 * 
 */
float readFloatSerialNext(unsigned char *ucTrame, int *offset)
{
  float destination;
  int i=0;
  unsigned char octet;
			
  for(i=0;i<4;i++)
    {
      ((char*)&destination)[i] = readUcharSerialNext(ucTrame, offset);
   	 
#ifdef DEBUG_FLOAT  
      printf("FLOAT OCTET %d = %#0x\n", i, octet); 
#endif
    }
	
#ifdef DEBUG_FLOAT
  printf("RESULTAT FLOAT =>  %.5f\n",destination);
  printf("  offset = %d\n", *offset);
#endif	

  return destination;	
}
	
