/*!
 * Copyright (c) 2006 Frederic Camps
 *
 * $Source:
 * $Revision: 0.a
 * $Date: 24 april 2006
 * 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "serial.h"
#include "sampleAPI.h"

extern int optind;
extern char *optarg;



/*! 
  @brief
 Utilisation du programme principal
*/
static void usage(const char *progname)
{
  fprintf(stderr, "usage: %s [-v] serial_device [-l] logFile\n", progname);
  exit(1);
}


/*!
 @brief
 Main du programme principal
 */
int startGPS(int argc, char *argv[])
{
  int d, i, fd, n, nSent, ch;
  char *prog = argv[0];

  int nbTrame;

  static char *logFile=NULL;
  static int modeVerbose=0;
		
  while ((ch = getopt(argc, argv, "vl:")) != -1)
    {
      switch (ch)
	{
	case 'v':
	  modeVerbose=1;
	  break;

	case 'l':
	  logFile = optarg;
	  modeVerbose=2;
	  break;

	case '?':
	default:
	  usage(argv[0]);
	} /* switch */
    }

  argc -= optind;
  argv += optind;
  
  if (argc < 1)
    {
      usage(prog);
    }
  
  // Ouverture du port serie en mode non bloquant avec une 
  // attente de 400ms et pas de contrainte sur la quantit de
  // donnes  lire
  if( (fd = openSerial(argv[0], 0, 4, 57600)) == -1) { exit (1); }

  // Initialise le mode verbeux et/ou des logs dans un fichier
  setLogData(modeVerbose, 0, logFile);


  // Initialisation du GPS
  //
  // Config des COM

  char CONF_COM1[]="COM COM1 57600\n";
  writeSerial(fd,  CONF_COM1);

  char CONF_COM2[]="COM COM2 57600\n";
  writeSerial(fd,  CONF_COM2);

  char INTERFACEMODE_COM2[]="INTERFACEMODE COM2 CMR NOVATEL\n";
  writeSerial(fd, INTERFACEMODE_COM2);

  // Configuration des LOGS

  char UNLOG_ALL_COM1[]="UNLOGALL COM1 TRUE\n"; // COM2 recoit les log de la base
  writeSerial(fd,  UNLOG_ALL_COM1);

  char LOG_BESTXYZb_COM1[]="LOG COM1 BESTXYZb ontime 1\n"; 
//  writeSerial(fd, LOG_BESTXYZb_COM1);

  char LOG_BESTPOSb_COM1[]="LOG COM1 BESTPOSb ontime 1\n"; 
//  writeSerial(fd, LOG_BESTPOSb_COM1);

  char LOG_BESTUTMb_COM1[]="LOG COM1 BESTUTMb ontime 0.5\n"; 
  writeSerial(fd, LOG_BESTUTMb_COM1);
 

  // Lecture port serie	
  while(1)
    {
      readSerial(fd, &nbTrame);	 

      //sleep(3);
    } 	
  exit(0);
}



/*! 
  @brief Initialisation du GPS par un programme externe - > module Genome
  

 @param 
*/
int initGPS(char *logFile, int modeVerbose, char *serialCOM, int VMIN, int VTIME, int serialSpeed)
{
  int fd;

  // Ouverture du port serie en mode non bloquant avec VTIME et VTIME 
  if( (fd = openSerial(serialCOM, VMIN, VTIME, serialSpeed)) == -1) { return (-1); }
  
  // Initialise le mode verbeux et/ou des logs dans un fichier
  setLogData(modeVerbose, 0, logFile);
  
  return fd;
}


		

/*! 
  @brief Lecture des donnes GPS par un prog externe -> module Genome
  

 @param 
*/
int getGPSData(int fd, int* nbTrame)
{
 return  readSerial(fd, nbTrame);    
}
